==> wc3maptranslator-3.0.2.tgz: package/test/TranslatorReversion.ts <==
import assert from 'assert';
import * as diff from 'diff-buf';
import * as fs from 'fs-extra';
import * as Path from 'path';

import { Translator } from '../index';

const testTranslator = new Translator();
const ObjectType = testTranslator.Objects.ObjectType;
const war3mapDir = Path.resolve('examples/warToJson/files/input');
const outputDir = Path.resolve('test/.output');

function readWar3MapBuffer(filename: string) {
    return fs.readFileSync(Path.join(war3mapDir, filename));
}

function buffersAreEqual(b1: Buffer, b2: Buffer) {
    const comparison = diff.default(b1, b2, { string: false });

    // Library `diff` returns an array of objects documenting comparison
    // e.g. { added: undefined, removed: undefined, value: 10 }
    // We want all `added` and `removed` fields to be "undefined" for
    // the buffers to be considered equal
    let buffersEqual = true;
    comparison.forEach((compare) => {
        if (compare.added || compare.removed) {
            buffersEqual = false;
        }
    });

    return buffersEqual;
}

// Ensures that when a war3map file is converted to JSON and back again,
// the two war3map files are the same; converting between the two data formats
// should be a reversible process
describe('Reversion: war -> json -> war', () => {

    before(() => {
        fs.emptyDirSync(outputDir);
        fs.ensureDirSync(outputDir);
    });

    it('Doodads reversion', () => {
        // Take war3map.doo, -> JSON, -> war3map.doo
        // Compare buffers, ensure they are equal
        const originalBuffer = readWar3MapBuffer('war3map.doo');
        const result = testTranslator.Doodads.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Doodads.jsonToWar(result.json).buffer;
        fs.writeFileSync(Path.join(outputDir, 'war3map.doo'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Strings reversion', () => {
        // this one is pending the strings translator support for war3map -> json
        assert(false, 'Not implemented');
    });

    it('Terrain reversion', () => {
        // take war3map.w3e -> json -> war3map.w3e
        const originalBuffer = readWar3MapBuffer('war3map.w3e');
        const result = testTranslator.Terrain.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Terrain.jsonToWar(result.json).buffer;
        fs.writeFileSync(Path.join(outputDir, 'war3map.w3e'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Units reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3mapUnits.doo');
        const result = testTranslator.Units.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Units.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3mapUnits.doo'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Regions reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3r');
        const result = testTranslator.Regions.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Regions.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3r'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Cameras reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3c');
        const result = testTranslator.Cameras.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Cameras.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3c'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Sounds reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3s');
        const result = testTranslator.Sounds.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Sounds.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3s'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Units (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3u');
        const result = testTranslator.Objects.warToJson(ObjectType.Units, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Units, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3u'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Items (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3t');
        const result = testTranslator.Objects.warToJson(ObjectType.Items, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Items, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3t'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Destructables (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3b');
        const result = testTranslator.Objects.warToJson(ObjectType.Items, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Items, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3b'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Doodads (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3d');
        const result = testTranslator.Objects.warToJson(ObjectType.Doodads, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Doodads, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3d'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Abilities (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3a');
        const result = testTranslator.Objects.warToJson(ObjectType.Abilities, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Abilities, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3a'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Buffs (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3h');
        const result = testTranslator.Objects.warToJson(ObjectType.Buffs, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Buffs, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3h'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Upgrades (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3q');
        const result = testTranslator.Objects.warToJson(ObjectType.Upgrades, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Upgrades, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3q'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Info reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3i');
        const result = testTranslator.Info.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Info.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3i'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Imports reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.imp');
        const result = testTranslator.Imports.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Imports.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.imp'), translatedBuffer);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

});

==> wc3maptranslator-3.0.3.tgz: package/test/TranslatorReversion.ts <==
import assert from 'assert';
import * as diff from 'diff-buf';
import * as fs from 'fs-extra';
import * as Path from 'path';

import { Translator } from '../index';

const testTranslator = new Translator();
const ObjectType = testTranslator.Objects.ObjectType;
const war3mapDir = Path.resolve('examples/war');
const outputDir = Path.resolve('test/.output');

function readWar3MapBuffer(filename: string) {
    return fs.readFileSync(Path.join(war3mapDir, filename));
}

function buffersAreEqual(b1: Buffer, b2: Buffer) {
    const comparison = diff.default(b1, b2, { string: false });

    // Library `diff` returns an array of objects documenting comparison
    // e.g. { added: undefined, removed: undefined, value: 10 }
    // We want all `added` and `removed` fields to be "undefined" for
    // the buffers to be considered equal
    let buffersEqual = true;
    comparison.forEach((compare) => {
        if (compare.added || compare.removed) {
            buffersEqual = false;
        }
    });

    return buffersEqual;
}

function writeJsonFile(fileName: string, json: any) {
    fs.writeFileSync(
        Path.join(outputDir, fileName),
        JSON.stringify(json, null, 4)
    );
}

// Ensures that when a war3map file is converted to JSON and back again,
// the two war3map files are the same; converting between the two data formats
// should be a reversible process
describe('Reversion: war -> json -> war', () => {

    before(() => {
        fs.emptyDirSync(outputDir);
        fs.ensureDirSync(outputDir);
    });

    it('Doodads reversion', () => {
        // Take war3map.doo, -> JSON, -> war3map.doo
        // Compare buffers, ensure they are equal
        const originalBuffer = readWar3MapBuffer('war3map.doo');
        const result = testTranslator.Doodads.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Doodads.jsonToWar(result.json).buffer;
        fs.writeFileSync(Path.join(outputDir, 'war3map.doo'), translatedBuffer);
        writeJsonFile('doodads.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Strings reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.wts');
        const result = testTranslator.Strings.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Strings.jsonToWar(result.json).buffer;
        fs.writeFileSync(Path.join(outputDir, 'war3map.wts'), translatedBuffer);
        writeJsonFile('strings.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Terrain reversion', () => {
        // take war3map.w3e -> json -> war3map.w3e
        const originalBuffer = readWar3MapBuffer('war3map.w3e');
        const result = testTranslator.Terrain.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Terrain.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3e'), translatedBuffer);
        writeJsonFile('terrain.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Units reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3mapUnits.doo');
        const result = testTranslator.Units.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Units.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3mapUnits.doo'), translatedBuffer);
        writeJsonFile('units.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Regions reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3r');
        const result = testTranslator.Regions.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Regions.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3r'), translatedBuffer);
        writeJsonFile('regions.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Cameras reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3c');
        const result = testTranslator.Cameras.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Cameras.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3c'), translatedBuffer);
        writeJsonFile('cameras.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Sounds reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3s');
        const result = testTranslator.Sounds.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Sounds.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3s'), translatedBuffer);
        writeJsonFile('sounds.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Units (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3u');
        const result = testTranslator.Objects.warToJson(ObjectType.Units, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Units, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3u'), translatedBuffer);
        writeJsonFile('object-units.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Items (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3t');
        const result = testTranslator.Objects.warToJson(ObjectType.Items, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Items, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3t'), translatedBuffer);
        writeJsonFile('object-items.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Destructables (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3b');
        const result = testTranslator.Objects.warToJson(ObjectType.Items, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Items, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3b'), translatedBuffer);
        writeJsonFile('object-destructables.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Doodads (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3d');
        const result = testTranslator.Objects.warToJson(ObjectType.Doodads, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Doodads, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3d'), translatedBuffer);
        writeJsonFile('object-doodads.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Abilities (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3a');
        const result = testTranslator.Objects.warToJson(ObjectType.Abilities, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Abilities, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3a'), translatedBuffer);
        writeJsonFile('object-abilities.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Buffs (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3h');
        const result = testTranslator.Objects.warToJson(ObjectType.Buffs, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Buffs, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3h'), translatedBuffer);
        writeJsonFile('object-buffs.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Upgrades (Object) reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3q');
        const result = testTranslator.Objects.warToJson(ObjectType.Upgrades, originalBuffer);
        const translatedBuffer = testTranslator.Objects.jsonToWar(ObjectType.Upgrades, result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3q'), translatedBuffer);
        writeJsonFile('object-upgrades.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Info reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.w3i');
        const result = testTranslator.Info.warToJson(originalBuffer);
        console.log(JSON.stringify(result.json, null, 4));
        const translatedBuffer = testTranslator.Info.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.w3i'), translatedBuffer);
        writeJsonFile('info.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

    it('Imports reversion', () => {
        const originalBuffer = readWar3MapBuffer('war3map.imp');
        const result = testTranslator.Imports.warToJson(originalBuffer);
        const translatedBuffer = testTranslator.Imports.jsonToWar(result.json).buffer;

        fs.writeFileSync(Path.join(outputDir, 'war3map.imp'), translatedBuffer);
        writeJsonFile('imports.json', result.json);

        assert(buffersAreEqual(originalBuffer, translatedBuffer));
    });

});
